package be.iminds.ilabt.jfed.espec;

import be.iminds.ilabt.jfed.espec.model.*;
import be.iminds.ilabt.jfed.espec.parser.ESpecConstants;
import be.iminds.ilabt.jfed.espec.parser.ESpecDefaults;

import javax.annotation.Nonnull;
import java.util.ArrayList;
import java.util.List;

/**
 * ESpecLogic
 */
public class ESpecLogic {
    private ESpecLogic() { throw new RuntimeException("static helper class"); }

    public static boolean hasAnsibleNode(@Nonnull ExperimentSpecification espec) {
        return espec.getAnsible() != null &&
                espec.getAnsible().getAnsibleHostSpec().getHostType() != AnsibleHostSpec.HostType.LOCAL;
    }

    public static boolean isAnsibleSubPart(@Nonnull UploadLikeSpec uploadLikeSpec,
                                           @Nonnull ExperimentSpecification espec) {
        if (espec.getAnsible() == null) return false;
        if (uploadLikeSpec instanceof AnsiblePlaybookSpec) return true;
        if (uploadLikeSpec instanceof AnsibleGalaxySpec) return true;
        if (uploadLikeSpec instanceof AnsiblePlaybookSpec.ExtraVarsJsonFileSpec) return true;

        List<UploadLikeSpec> ansibleUploads = new ArrayList<>();
        ansibleUploads.addAll(espec.getAnsible().getAnsibleHostSpec().getUploadLists());
        ansibleUploads.addAll(espec.getAnsible().getAnsibleHostSpec().getExecuteLists());
        ansibleUploads.addAll(espec.getAnsible().getAnsibleGalaxySpecs());
        ansibleUploads.addAll(espec.getAnsible().getAnsiblePlaybookSpecs());

        for (UploadLikeSpec ansibleUploadLikeSpec : ansibleUploads) {
            //this EXPLICITLY needs to use real equality, just to be certain.
            //(and yes, this loop is not efficient, but that shouldn't be a problem here)
            //noinspection ObjectEquality
            if (ansibleUploadLikeSpec == uploadLikeSpec) {
                return true;
            }
        }
        return false;
    }

    public static boolean isAnsibleNode(@Nonnull String clientId,
                                        @Nonnull ExperimentSpecification espec) {
        if (espec.getAnsible() == null) return false;
        if (espec.getAnsible().getAnsibleHostSpec().getHostType() == AnsibleHostSpec.HostType.LOCAL) return false;

        String ansibleNodeName = espec.getAnsible().getAnsibleHostSpec().getNodeName() == null ?
                ESpecDefaults.ANSIBLE_NODE_NAME :
                espec.getAnsible().getAnsibleHostSpec().getNodeName();
        return clientId.equals(ansibleNodeName);
    }

    /**
     * This assumes there IS an ansible section in the ExperimentSpecification
     * @param experimentSpecification
     * @return
     */
    @Nonnull
    public static String getAnsibleNodeName(@Nonnull ExperimentSpecification experimentSpecification) {
        if (experimentSpecification.getAnsible() == null)
            throw new IllegalStateException("Ansible section in ExperimentSpecification was assumed but not found");
        String ansibleNodeName = experimentSpecification.getAnsible().getAnsibleHostSpec().getNodeName();
        if (ansibleNodeName == null) return ESpecDefaults.ANSIBLE_NODE_NAME;
        return ansibleNodeName;
    }

    public static boolean mustRunOn(@Nonnull String clientId,
                                    @Nonnull ExperimentSpecification espec,
                                    @Nonnull DirSpec dirSpec) {
        if (dirSpec.getNodes() == null) {
            if (dirSpec.isAnsibleContent())
                return isAnsibleNode(clientId, espec);
            else
                return espec.getConfig().isIncludeAnsibleNodeInAllNodes() || !isAnsibleNode(clientId, espec);
        } else
            return dirSpec.getNodes().contains(clientId);

    }
    public static boolean mustRunOn(@Nonnull String clientId,
                                    @Nonnull ExperimentSpecification espec,
                                    @Nonnull UploadLikeSpec aSpec) {
        if (isAnsibleNode(clientId, espec))
            return isAnsibleSubPart(aSpec, espec);
        else {
            if (aSpec.getNodes() != null)
                return aSpec.getNodes().contains(clientId);

            return espec.getConfig().isIncludeAnsibleNodeInAllNodes() || !isAnsibleSubPart(aSpec, espec);
        }
    }
}
